%% TPK4560 - Project-calculations
 % Author:  Aslak J. Strand
 % Purpose: Testing MPU6050 with RPi and Arduino
%% Clearing workspace:
clear all; close all; clc
pause on

%% Test
board = input('[1] - RPi\n[2] - Arduino\nChoose your board: ');
%board = 2;
sample = 0.1;
runTime = input('Run time [s]: ');

acc = figure('Name','Accelerometer');
axl = animatedline('Color','r');
ayl = animatedline('Color','b');
azl = animatedline('Color','g');
legend('Ax','Ay','Az')
%xlim([0 runTime/sample])
xlabel('Time [s]');
ylabel('m/s2');
figure(acc)

gyr = figure('Name','Gyro');
gxl = animatedline('Color','r');
gyl = animatedline('Color','b');
gzl = animatedline('Color','g');
legend('Gx','Gy','Gz')
%xlim([0 runTime/sample])
xlabel('Time [s]');
ylabel('°/s');
figure(gyr)


%% Selecting Board to use
switch board
    %% SSH connection to RPi
    case 1
        fprintf('\nUsing RaspberryPi\n\n')

        IP = input('Enter IP-address: ', 's');
        if IP == ""
            fprintf('Using saved setup\n')
            load('rpiVal.mat');
        else
            UN = input('Enter UserName:   ', 's');
            PW = input('Enter PassWord:   ', 's');
            save('rpiVal.mat', 'IP', 'UN', 'PW');
        end
        fprintf('\nEstablishing SSH connection to:\nRPi @ %s\n', IP)

        try
            rpi = raspi(IP, UN, PW);
        catch errno
            fprintf('\nError: %s\nExitin program.\n\n',errno.identifier)
            clear IMU
            clear rpi
            return
        end

        enableI2C(rpi);
        fprintf('i2c @ %s\n\n', rpi.AvailableI2CBuses{1})

        BUS = scanI2CBus(rpi, rpi.AvailableI2CBuses{1});
        if isempty(BUS)
            fprintf('No I2C devices connected to the RPi.\nExiting program.\n')
            clear IMU
            clear rpi
            return
        end
        IMU = i2cdev(rpi, rpi.AvailableI2CBuses{1}, BUS{1});

        % INIT
        write(IMU,hex2dec('19'),'int8') % sample rate register
        write(IMU,hex2dec('6B'),'int8') % power management
        write(IMU,hex2dec('1A'),'int8') % Config
        write(IMU,hex2dec('1B'),'int8') % Gyro Config
        write(IMU,hex2dec('38'),'int8') % Gyro Config

        if isnan(runTime)
            load('rpiVal.mat', 'runTime')
        else
            save('rpiVal.mat', 'runTime', '-append');
        end

        Ax = [zeros(1,round(runTime*3)+1)];
        Ay = [zeros(1,round(runTime*3)+1)];
        Az = [zeros(1,round(runTime*3)+1)];
        Gx = [zeros(1,round(runTime*3)+1)];
        Gy = [zeros(1,round(runTime*3)+1)];
        Gz = [zeros(1,round(runTime*3)+1)];
        figTime = [zeros(1,round(runTime*3)+1)];

        time = clock;
        stop = time;

        if (stop(6) + runTime) > 60
            stop(5) = stop(5) + round( (stop(6) + runTime) / 60 );
            stop(6) = stop(6) + runTime - 60;
        else
            stop(6) = stop(6) + runTime;
        end

        i = 1;
        while (etime(stop,time)) >= seconds(0)
            figTime(i) = i;
            fprintf('Sample #%i\ttime: %2.0f:%2.0f \t stop @ %2.0f:%2.0f\n',i,time(5),time(6),stop(5),stop(6))
            % Accelerometer RAW:
            acc_x_H = readRegister(IMU, hex2dec('3B'),'int8');
            acc_x_L = readRegister(IMU, hex2dec('3C'),'int8');
            axValue = bitor(bitsll(acc_x_H, 8), acc_x_L);
            if axValue > 32768
                axValue = axValue - 65536;
            end

            acc_y_H = readRegister(IMU, hex2dec('3D'),'int8');
            acc_y_L = readRegister(IMU, hex2dec('3E'),'int8');
            ayValue = bitor(bitsll(acc_y_H, 8), acc_y_L);
            if ayValue > 32768
                ayValue = ayValue - 65536;
            end

            acc_z_H = readRegister(IMU, hex2dec('3F'),'int8');
            acc_z_L = readRegister(IMU, hex2dec('40'),'int8');
            azValue = bitor(bitsll(acc_z_H, 8), acc_z_L);
            if azValue > 32768
                azValue = azValue - 65536;
            end

            % Gyro RAW:
            gyro_x_H = readRegister(IMU, hex2dec('43'),'int8');
            gyro_x_L = readRegister(IMU, hex2dec('44'),'int8');
            gxValue = bitor(bitsll(gyro_x_H, 8), gyro_x_L);
            if gxValue > 32768
                gxValue = gxValue - 65536;
            end

            gyro_y_H = readRegister(IMU, hex2dec('45'),'int8');
            gyro_y_L = readRegister(IMU, hex2dec('44'),'int8');
            gyValue = bitor(bitsll(gyro_y_H, 8), gyro_y_L);
            if gyValue > 32768
                gyValue = gyValue - 65536;
            end

            gyro_z_H = readRegister(IMU, hex2dec('47'),'int8');
            gyro_z_L = readRegister(IMU, hex2dec('44'),'int8');
            gzValue = bitor(bitsll(gyro_z_H, 8), gyro_z_L);
            if gzValue > 32768
                gzValue = gzValue - 65536;
            end


            % Fix
            Ax(i) = double(axValue / 16384.0);
            Ay(i) = double(ayValue / 16384.0);
            Az(i) = double(azValue / 16384.0);

            Gx(i) = double(gxValue / 131.0);
            Gy(i) = double(gyValue / 131.0);
            Gz(i) = double(gzValue / 131.0);

            % Print
            fprintf('Gx=%.2f °/s\t Gy=%.2f °/s\t Gy=%.2f °/s\n',Gx(i),Gy(i),Gz(i))
            fprintf('Ax=%.2f g\t Ay=%.2f g\t Az=%.2f g\n\n',Ax(i),Ay(i),Az(i))


            figure(acc)
            hold on
            addpoints(axl, figTime(i),Ax(i));
            addpoints(axl, figTime(i),Ay(i));
            addpoints(axl, figTime(i),Az(i));
            hold off

            figure(gyr)
            hold on
            addpoints(gxl, figTime(i), Gx(i))
            addpoints(gyl, figTime(i), Gy(i))
            addpoints(gzl, figTime(i), Gz(i))
            hold off



            pause(sample)
            time = clock;
            i = i + 1;
        end
        clear IMU
        clear rpi
        return

    case 2
        %% USB connection to ARDUINO
        fprintf('\nUsing Arduino\n\n')
        
        fprintf('~Trying to connect to Arduino~\n')
        try
            ard = arduino;
            
            fprintf('Connected to ARDUINO @ %s\n\n', ard.Port)
        catch errno
            fprintf('Failed to connect to ARDUINO with error:\n%s\n',errno.identifier)
            fprintf('Try to run <<arduinosetup>> from console.')
            return
        end
        
        fprintf('~Trying to connect to IMU~\n')
        try
            imu = mpu6050(ard,'SampleRate',50,'SamplesPerRead',1,'ReadMode','Latest');
            
            fprintf('Connected to IMU address: 0x%s\n\n', dec2hex(imu.I2CAddress))
            
        catch errno
            fprintf('Failed to connect to IMU with error:\n%s\n',errno.identifier)
            return
        end
        
        time = clock;
        strt = time;
        stop = time;

        if (stop(6) + runTime) > 60
            stop(5) = stop(5) + round( (stop(6) + runTime) / 60 );
            stop(6) = stop(6) + runTime - 60;
        else
            stop(6) = stop(6) + runTime;
        end

        
        i = 1;
        while (etime(stop,time)) >= seconds(0)
            %fprintf('Sample #%i\ttime: %2.0f:%2.0f \t stop @ %2.0f:%2.0f\n',i,time(5),time(6),stop(5),stop(6))
            if ( rem(i,30) == 0 )
                fprintf('Sample #%i\ttime: %2.0f:%2.0f \t stop @ %2.0f:%2.0f\n',i,time(5),time(6),stop(5),stop(6))
            end
            %Read raw data & split
            [sensorReadings,overrun] = read(imu);

            Acc_Vals = sensorReadings.Acceleration;
            for a = 1:3
                if Acc_Vals(a) > 32768
                    Acc_Vals(a) = Acc_Vals(a) - 65536;
                end
            end

            Gyr_Vals = sensorReadings.AngularVelocity;
            for g = 1:3
                if Gyr_Vals(g) > 32768
                    Gyr_Vals(g) = Gyr_Vals(g) - 65536;
                end
            end
            
            

            % Fix
            Ax(i) = double(Acc_Vals(1));
            Ay(i) = double(Acc_Vals(2));
            Az(i) = double(Acc_Vals(3));
            % / 16384.0
            Gx(i) = double(Gyr_Vals(1)*(180/pi));
            Gy(i) = double(Gyr_Vals(2)*(180/pi));
            Gz(i) = double(Gyr_Vals(3)*(180/pi)); 
            % / 131.0);


            % Print
            %fprintf('Gx=%.2f °/s\t Gy=%.2f °/s\t Gy=%.2f °/s\n',Gx(i),Gy(i),Gz(i))
            %fprintf('Ax=%.4f g\t Ay=%.4f g\t Az=%.4f g\n\n',Ax(i),Ay(i),Az(i))

            t(i) = etime(time, strt);
            
            figure(acc)
            hold on
            addpoints(axl, t(i), Ax(i));
            addpoints(ayl, t(i), Ay(i));
            addpoints(azl, t(i), Az(i));
            hold off

            figure(gyr)
            hold on
            addpoints(gxl, t(i), Gx(i))
            addpoints(gyl, t(i), Gy(i))
            addpoints(gzl, t(i), Gz(i))
            hold off


            pause(sample)
            time = clock;
            i = i+1;
        end
        return
end
